==> mongoop-0.1.5.tgz: package/index.js <==
/*
 *  mongoop
 *  Copyright(c) 2017 Giles Aamoth
 *  MIT Licensed
*/
'use strict'

var mongoose = require('mongoose')

module.exports = mongoop

function mongoop(uri) {
  let connection = mongoose.connect(uri)

  // handle any errors while trying to connect to database
  let errorhandler = connection.connection.on('error', err => console.error(err))

  // object that will be returned by function
  let mongoopObject = {
    collections: collections
  }

  // add collections to mongoopObject
  function collections(options) {
    if (Array.isArray(options)) { // multiple collections
      options.forEach(e => {
        let query = setup(e)
        let opers = operations(query)
        init(e.model, opers)
      })
    }
    else if (Array.isArray(options) && typeof options === 'object') { // a single collection
      let query = setup(options)
      let opers = operations(query)
      init(options.model, opers)
    }
  }

  // setup a collection
  function setup(collection) {
    let queryschema = null
    let query = null

    queryschema = new mongoose.Schema(collection.schema)
    queryschema = sethooks(queryschema, collection.hooks)
    //queryschema = setplugins(queryschema)

    query = connection.model(collection.model, queryschema)
    
    return query
  }

  // expose CRUD operations for a collection
  function init(model, operations) {
    mongoopObject[model] = {
      operations: operations
    }
  }

  // returns object containing CRUD operations on a collection
  function operations(Query) {
    return {
      create (options, callback) {
        let query = new Query(options)

        query.save(err => {
          if (err) {
            return callback(err)
          }
          callback(null, options)
        })
      },
      read (querymethod, options, callback) {
        Query[querymethod](options, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      update (query, update, callback) {
        Query.update(query, update, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      delete (query, callback) {
        Query.remove(query, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      }
    }
  }

  // accepts single hook assignments (array)
  // accepts multiple hook assignments (array of arrays)
  // TODO: add functionality for running hooks in series or parallel
  function sethooks(schema, hooks) {
    hooks.forEach(e => {
      if (e[0] === 'pre') {
        schema = prehook(schema, e[1], e[2])
      }
      else {
        schema = posthook(schema, e[1], e[2])
      }
    })

    return schema
  }

  // assign a prehook to schema
  function prehook(schema, action, callback) {
    return schema.pre(action, callback)
  }

  // assign a posthook to schema
  function posthook(schema, action, callback) {
    return schema.post(action, callback)
  }

  // setplugins -- noop
  function setplugins() {}

  return mongoopObject
}

==> mongoop-0.1.6.tgz: package/index.js <==
/*
 *  mongoop
 *  Copyright(c) 2017 Giles Aamoth
 *  MIT Licensed
*/
'use strict'

var mongoose = require('mongoose')

module.exports = mongoop

function mongoop(uri) {
  let connection = mongoose.connect(uri)

  // handle any errors while trying to connect to database
  let errorhandler = connection.connection.on('error', err => console.error(err))

  // object that will be returned by function
  let mongoopObject = {
    collections: collections
  }

  // add collections to mongoopObject
  function collections(options) {
    if (Array.isArray(options)) { // multiple collections
      options.forEach(e => {
        let query = setup(e)
        let opers = operations(query)
        init(e.model, opers)
      })
    }
    else if (!Array.isArray(options) && typeof options === 'object') { // a single collection
      let query = setup(options)
      let opers = operations(query)
      init(options.model, opers)
    }
  }

  // setup a collection
  function setup(collection) {
    let queryschema = null
    let query = null

    queryschema = new mongoose.Schema(collection.schema)
    queryschema = sethooks(queryschema, collection.hooks)
    //queryschema = setplugins(queryschema)

    query = connection.model(collection.model, queryschema)
    
    return query
  }

  // expose CRUD operations for a collection
  function init(model, operations) {
    mongoopObject[model] = {
      operations: operations
    }
  }

  // returns object containing CRUD operations on a collection
  function operations(Query) {
    return {
      create (options, callback) {
        let query = new Query(options)

        query.save(err => {
          if (err) {
            return callback(err)
          }
          callback(null, options)
        })
      },
      read (querymethod, options, callback) {
        Query[querymethod](options, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      update (query, update, callback) {
        Query.update(query, update, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      delete (query, callback) {
        Query.remove(query, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      }
    }
  }

  // accepts single hook assignments (array)
  // accepts multiple hook assignments (array of arrays)
  // TODO: add functionality for running hooks in series or parallel
  function sethooks(schema, hooks) {
    hooks.forEach(e => {
      if (e[0] === 'pre') {
        schema = prehook(schema, e[1], e[2])
      }
      else {
        schema = posthook(schema, e[1], e[2])
      }
    })

    return schema
  }

  // assign a prehook to schema
  function prehook(schema, action, callback) {
    return schema.pre(action, callback)
  }

  // assign a posthook to schema
  function posthook(schema, action, callback) {
    return schema.post(action, callback)
  }

  // setplugins -- noop
  function setplugins() {}

  return mongoopObject
}
